@@ -39,7 +39,7 @@ from rattail.util import (load_object, load_entry_points,
 from rattail.files import temp_path
 from rattail.mail import send_email
 from rattail.config import parse_list
-from rattail.core import get_uuid
+from rattail.core import get_uuid, Object
 
 
 log = logging.getLogger(__name__)
@@ -614,6 +614,22 @@ class AppHandler(object):
         if require:
             raise ValueError("Cannot locate handler spec for key: {}".format(key))
 
+    def get_label_handler(self, **kwargs):
+        """
+        Get the configured "label" handler.
+
+        See also :doc:`rattail-manual:base/handlers/other/labels`.
+
+        :returns: The :class:`~rattail.labels.LabelHandler` instance
+           for the app.
+        """
+        if not hasattr(self, 'label_handler'):
+            spec = self.config.get('rattail', 'labels.handler',
+                                   default='rattail.labels:LabelHandler')
+            factory = self.load_object(spec)
+            self.label_handler = factory(self.config, **kwargs)
+        return self.label_handler
+
     def get_membership_handler(self, **kwargs):
         """
         Get the configured "membership" handler.
@@ -721,6 +737,13 @@ class AppHandler(object):
         """
         return progress_loop(*args, **kwargs)
 
+    def make_object(self, **kwargs):
+        """
+        Create and return a generic object.  All kwargs will be
+        assigned as attributes to the object.
+        """
+        return Object(**kwargs)
+
     def get_session(self, obj):
         """
         Returns the SQLAlchemy session with which the given object is
