@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -342,12 +342,8 @@ class CustomerOrderBatchHandler(BatchHandler):
         batch.email_address = pending.email_address
 
     def get_case_size_for_product(self, product):
-        if product.case_size:
-            return product.case_size
-
-        cost = product.cost
-        if cost:
-            return cost.case_size
+        products = self.app.get_products_handler()
+        return products.get_case_size(product)
 
     def get_case_price_for_row(self, row):
         """
