@@ -232,6 +232,11 @@ class PricingBatchHandler(BatchHandler):
         """
         Set the row's status code according to its price diff
         """
+        # manually priced items are "special" unless batch says to re-calc
+        if row.manually_priced and not row.batch.calculate_for_manual:
+            row.status_code = row.STATUS_PRODUCT_MANUALLY_PRICED
+            return
+
         # prefer "% Diff" if batch defines that
         threshold = row.batch.min_diff_percent
         if threshold:
