@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -299,11 +299,15 @@ class DataSyncImportConsumer(DataSyncConsumer):
         Attempt to invoke the importer, to delete a local record according to
         the change involved.
         """
-        key = self.get_deletion_key(session, change)
-        if key is not None:
-            local_object = importer.get_local_object(key)
-            if local_object and importer.allow_delete:
-                return importer.delete_object(local_object)
+        if importer.allow_delete:
+            key = self.get_deletion_key(session, change)
+            if key is not None:
+                local_object = importer.get_local_object(key)
+                if local_object:
+                    local_data = importer.normalize_local_object(local_object)
+                    if local_data and importer.can_delete_object(
+                            local_object, local_data):
+                        return importer.delete_object(local_object)
         return False
 
     def get_deletion_key(self, session, change):
