@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -28,6 +28,7 @@ from __future__ import unicode_literals, absolute_import
 
 import datetime
 
+import sqlalchemy as sa
 from sqlalchemy import orm
 
 from rattail.db import api
@@ -125,3 +126,28 @@ def next_batch_id(session):
     """
     sql = "select nextval('datasync_change_batch_id_seq')"
     return session.execute(sql).scalar()
+
+
+def purge_datasync_settings(config, session):
+    model = config.get_model()
+
+    to_delete = [
+        'rattail.datasync.watch',
+        'tailbone.datasync.restart',
+    ]
+    for setting in to_delete:
+        setting = session.query(model.Setting).get(setting)
+        if setting:
+            session.delete(setting)
+
+    session.query(model.Setting)\
+           .filter(sa.or_(
+               model.Setting.name.like('rattail.datasync.%.watcher'),
+               model.Setting.name.like('rattail.datasync.%.watcher.db'),
+               model.Setting.name.like('rattail.datasync.%.watcher.delay'),
+               model.Setting.name.like('rattail.datasync.%.watcher.retry_attempts'),
+               model.Setting.name.like('rattail.datasync.%.watcher.retry_delay'),
+               model.Setting.name.like('rattail.datasync.%.consumers'),
+               model.Setting.name.like('rattail.datasync.%.consumers.runas'),
+               model.Setting.name.like('rattail.datasync.%.consumer.%')))\
+           .delete(synchronize_session=False)
