@@ -56,11 +56,19 @@ class VendorCatalogBatch(FileBatchMixin, Base):
     The key of the parser used to parse the contents of the data file.
     """)
 
-    vendor_uuid = sa.Column(sa.String(length=32), nullable=False)
+    vendor_uuid = sa.Column(sa.String(length=32), nullable=True)
     vendor = orm.relationship(Vendor, doc="""
     Reference to the :class:`Vendor` to which the catalog pertains.
     """)
 
+    vendor_id = sa.Column(sa.String(length=15), nullable=True, doc="""
+    ID string for the vendor.
+    """)
+
+    vendor_name = sa.Column(sa.String(length=50), nullable=True, doc="""
+    Name of the vendor.
+    """)
+
     future = sa.Column(sa.Boolean(), nullable=True, default=False, doc="""
     Flag indicating whether the batch should be treated as a "future cost"
     batch, as opposed to a "current" cost batch.  If set, the batch will create
@@ -98,6 +106,7 @@ class VendorCatalogBatchRow(ProductBatchRowMixin, Base):
     STATUS_CHANGE_VENDOR_ITEM_CODE = 5
     STATUS_CHANGE_CASE_SIZE = 6
     STATUS_CHANGE_COST = 7
+    STATUS_CHANGE_PRODUCT = 8
 
     STATUS = {
         STATUS_NO_CHANGE:               "no change",
@@ -107,6 +116,7 @@ class VendorCatalogBatchRow(ProductBatchRowMixin, Base):
         STATUS_CHANGE_VENDOR_ITEM_CODE: "change vendor item code",
         STATUS_CHANGE_CASE_SIZE:        "change case size",
         STATUS_CHANGE_COST:             "change cost",
+        STATUS_CHANGE_PRODUCT:          "change product",
     }
 
     cost_uuid = sa.Column(sa.String(length=32), nullable=True)
