@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -26,6 +26,8 @@ Problem Reports for Rattail Systems
 
 from __future__ import unicode_literals, absolute_import
 
+import datetime
+
 from sqlalchemy import orm
 
 from rattail.db import Session, model
@@ -66,6 +68,47 @@ class ProductWithoutPrice(RattailProblemReport):
         return problems
 
 
+class StaleInventoryBatch(RattailProblemReport):
+    """
+    Looks for "stale" inventory batches, those which were created but
+    not executed within a certain amount of time.
+    """
+    problem_key = 'stale_inventory_batch'
+    problem_title = "Stale inventory batches"
+
+    def __init__(self, *args, **kwargs):
+        super(StaleInventoryBatch, self).__init__(*args, **kwargs)
+
+        self.cutoff_days = self.config.getint(
+            'rattail', 'problems.stale_inventory_batches.cutoff_days',
+            default=4)
+
+    def find_problems(self, **kwargs):
+        session = self.app.make_session()
+        model = self.model
+
+        today = self.app.localtime().date()
+        cutoff = today - datetime.timedelta(days=self.cutoff_days)
+        cutoff = datetime.datetime.combine(cutoff, datetime.time(0))
+        cutoff = self.app.localtime(cutoff)
+
+        batches = session.query(model.InventoryBatch)\
+                         .filter(model.InventoryBatch.executed == None)\
+                         .filter(model.InventoryBatch.created <= self.app.make_utc(cutoff))\
+                         .options(orm.joinedload(model.InventoryBatch.created_by)\
+                                  .joinedload(model.User.person))\
+                         .all()
+
+        session.close()
+        return batches
+
+    def get_email_context(self, problems, **kwargs):
+        kwargs = super(StaleInventoryBatch, self).get_email_context(problems,
+                                                                    **kwargs)
+        kwargs['cutoff_days'] = self.cutoff_days
+        return kwargs
+
+
 class UpgradePending(RattailProblemReport):
     """
     Looks for any system upgrades which have yet to be executed.
