@@ -26,6 +26,9 @@ Products Handler
 
 from __future__ import unicode_literals, absolute_import
 
+import warnings
+import logging
+
 import six
 import sqlalchemy as sa
 from sqlalchemy import orm
@@ -38,6 +41,9 @@ from rattail.gpc import GPC
 from rattail.barcodes import upce_to_upca
 
 
+log = logging.getLogger(__name__)
+
+
 class ProductsHandler(GenericHandler):
     """
     Base class and default implementation for product handlers.
@@ -62,6 +68,15 @@ class ProductsHandler(GenericHandler):
            ``value`` is not valid, and ``ignore_errors`` was true,
            then returns ``None``.
         """
+        # normalize to string; remove unwanted chars
+        if not isinstance(value, six.string_types):
+            value = six.text_type(value)
+        value = value.replace(' ', '')
+        value = value.replace('-', '')
+
+        # auto-handle check digit unless specified
+        kwargs.setdefault('calc_check_digit', 'auto')
+
         try:
             return GPC(value, **kwargs)
         except:
@@ -107,6 +122,7 @@ class ProductsHandler(GenericHandler):
 
         return make_full_description(brand_name, description, size)
 
+    # TODO: deprecate / remove (afaik it is not being used?)
     def find_products_by_key(self, session, value, **kwargs):
         """
         Locate any products where the "key" matches the given value.
@@ -257,104 +273,315 @@ class ProductsHandler(GenericHandler):
            and sometimes those come through as integers etc.  If this
            value is a :class:`~rattail.gpc.GPC` instance, special
            logic may be used for the lookup.
+
+        :param lookup_fields: Optional list of fields to use for
+           lookup.  The default value is ``['uuid', '_product_key_']``
+           which means to lookup by UUID as well as "product key"
+           field, which is configurable.  You can include any of the
+           following in ``lookup_fields``:
+
+           * ``uuid``
+           * ``_product_key_`` - :meth:`locate_product_for_key`
+           * ``upc`` - :meth:`locate_product_for_upc`
+           * ``item_id`` - :meth:`locate_product_for_item_id`
+           * ``scancode`` - :meth:`locate_product_for_scancode`
+           * ``vendor_code`` - :meth:`locate_product_for_vendor_code`
+           * ``alt_code`` - :meth:`locate_product_for_alt_code`
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
         """
-        # don't bother if we're given empty "entry" value
+        model = self.model
         if not entry:
             return
 
         # if entry is GPC then only look for that type of match
+        # TODO: is this really the expected behavior?  maybe not..
         if isinstance(entry, GPC):
             return self.locate_product_for_gpc(session, entry)
 
-        # for the rest of this, assume entry is string
-        entry = six.text_type(entry)
-        model = self.model
+        # figure out which fields we should match on
+        # TODO: let config declare default lookup_fields
+        lookup_fields = kwargs.get('lookup_fields', [
+            'uuid',
+            '_product_key_',
+        ])
+        if kwargs.get('lookup_vendor_code'): # pragma: no cover
+            warnings.warn("`lookup_vendor_code` param is deprecated, "
+                          "please includle 'vendor_code' in `lookup_fields` "
+                          "instead", DeprecationWarning)
+            if 'vendor_code' not in lookup_fields:
+                lookup_fields.append('vendor_code')
+        if kwargs.get('lookup_by_code'): # pragma: no cover
+            warnings.warn("`lookup_by_code` param is deprecated, "
+                          "please includle 'alt_code' in `lookup_fields` "
+                          "instead", DeprecationWarning)
+            if 'alt_code' not in lookup_fields:
+                lookup_fields.append('alt_code')
 
         # try to locate product by uuid before other, more specific key
         # if kwargs.get('try_uuid', True):
-        product = session.query(model.Product).get(entry)
+        if 'uuid' in lookup_fields:
+            if isinstance(entry, six.string_types):
+                product = session.query(model.Product).get(entry)
+                if product:
+                    return product
+
+        lookups = {
+            'uuid': None,
+            '_product_key_': self.locate_product_for_key,
+            'upc': self.locate_product_for_upc,
+            'item_id': self.locate_product_for_item_id,
+            'scancode': self.locate_product_for_scancode,
+            'vendor_code': self.locate_product_for_vendor_code,
+            'alt_code': self.locate_product_for_alt_code,
+        }
+
+        for field in lookup_fields:
+            if field in lookups:
+                lookup = lookups[field]
+                if lookup:
+                    product = lookup(session, entry, **kwargs)
+                    if product:
+                        return product
+            else:
+                log.warning("unknown lookup field: %s", field)
+
+    def locate_product_for_upc(self, session, entry, **kwargs):
+        """
+        Locate the product which matches the given UPC value.
+
+        This will do a lookup on the
+        :attr:`rattail.db.model.products.Product.upc` field only.
+
+        The ``entry`` value provided is expected to be a string, which
+        will be coerced to a :class:`~rattail.gpc.GPC` value in
+        various ways while attempting the lookup.  If you already have
+        a known-good GPC value then you probably should be using
+        :meth:`locate_product_for_gpc()` instead.
+
+        Note that instead of calling this method directly, you might
+        consider calling :meth:`locate_product_for_key` instead.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: String value representing the UPC (or EAN13).
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
+        """
+        if not entry:
+            return
+
+        # assume entry is string; valid only if all digits
+        entry = six.text_type(entry)
+        if not entry.isdigit():
+            return
+
+        # we first assume the user entry *does* include check digit
+        provided = self.make_gpc(entry, calc_check_digit=False)
+        product = self.locate_product_for_gpc(session, provided, **kwargs)
         if product:
             return product
 
+        # but we can also calculate a check digit and try that
+        checked = self.make_gpc(entry, calc_check_digit='upc')
+        product = self.locate_product_for_gpc(session, checked, **kwargs)
+        if product:
+            return product
+
+        # one last trick is to expand UPC-E to UPC-A and then reattempt
+        # the lookup, *with* check digit (since it would be known)
+        if len(entry) in (6, 8):
+            checked = self.make_gpc(upce_to_upca(entry), calc_check_digit='upc')
+            product = self.locate_product_for_gpc(session, checked, **kwargs)
+            if product:
+                return product
+
+    def locate_product_for_item_id(self, session, entry, **kwargs):
+        """
+        Locate the product which matches the given item ID.
+
+        This will do a lookup on the
+        :attr:`rattail.db.model.products.Product.item_id` field only.
+
+        Note that instead of calling this method directly, you might
+        consider calling :meth:`locate_product_for_key` instead.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: Item ID value as string.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
+        """
+        from rattail.db.api.products import get_product_by_item_id
+
+        if not entry:
+            return
+
+        # assume entry is string
+        entry = six.text_type(entry)
+
+        # do basic lookup
+        return get_product_by_item_id(session, entry)
+
+    def locate_product_for_scancode(self, session, entry, **kwargs):
+        """
+        Locate the product which matches the given scancode.
+
+        This will do a lookup on the
+        :attr:`rattail.db.model.products.Product.scancode` field only.
+
+        Note that instead of calling this method directly, you might
+        consider calling :meth:`locate_product_for_key` instead.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: Scancode value as string.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
+        """
+        from rattail.db.api.products import get_product_by_scancode
+
+        if not entry:
+            return
+
+        # assume entry is string
+        entry = six.text_type(entry)
+
+        # do basic lookup
+        return get_product_by_scancode(session, entry)
+
+    def locate_product_for_key(self, session, entry, product_key=None, **kwargs):
+        """
+        Locate the product which matches the given key value.
+
+        This is an abstraction layer so calling logic need not care
+        which product key field is configured.  Under the hood this
+        will invoke one of:
+
+        * :meth:`locate_product_for_upc`
+        * :meth:`locate_product_for_item_id`
+        * :meth:`locate_product_for_scancode`
+
+        This will do a lookup on the product key field only.  It
+        normally checks config to determine which field to use for
+        product key (via
+        :meth:`~rattail.config.RattailConfig.product_key()`), but you
+        can override by specifying, e.g.  ``product_key='item_id'``.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: Key value to use for the lookup.
+
+        :param product_key: Optional key field to use for the lookup.
+           If not specified, will be read from config.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance if a match was found; otherwise ``None``.
+        """
         # prefer caller-provided key over configured key
-        product_key = kwargs.get('product_key')
         if not product_key:
             product_key = self.config.product_key()
 
+        product = None
+
         if product_key == 'upc':
-
-            if entry.isdigit():
-
-                # we first assume the user entry *does* include check digit
-                provided = GPC(entry, calc_check_digit=False)
-                product = api.get_product_by_upc(session, provided)
-                if product:
-                    return product
-
-                # but we can also calculate a check digit and try that
-                checked = GPC(entry, calc_check_digit='upc')
-                product = api.get_product_by_upc(session, checked)
-                if product:
-                    return product
-
-                # one last trick is to expand UPC-E to UPC-A and then reattempt
-                # the lookup, *with* check digit (since it would be known)
-                if len(entry) in (6, 8):
-                    checked = GPC(upce_to_upca(entry), calc_check_digit='upc')
-                    product = api.get_product_by_upc(session, checked)
-                    if product:
-                        return product
+            product = self.locate_product_for_upc(session, entry)
 
         elif product_key == 'item_id':
-
-            # try to locate product by item_id
-            product = api.get_product_by_item_id(session, entry)
-            if product:
-                return product
+            product = self.locate_product_for_item_id(session, entry)
 
         elif product_key == 'scancode':
+            product = self.locate_product_for_scancode(session, entry)
 
-            # try to locate product by scancode
-            product = api.get_product_by_scancode(session, entry)
-            if product:
-                return product
-
-        # if we made it this far, lookup by product key failed.
-
-        # try a vendor item code lookup, only if caller requests it
-        if kwargs.get('lookup_vendor_code'):
-            product = api.get_product_by_vendor_code(session, entry,
-                                                     vendor=kwargs.get('vendor'))
-            if product:
-                return product
-
-        # try an "alternate code" lookup, only if caller requests it
-        if kwargs.get('lookup_by_code'):
-            product = api.get_product_by_code(session, entry)
-            if product:
-                return product
+        return product
 
     def locate_product_for_gpc(self, session, gpc, **kwargs):
         """
-        Try to locate a product for the given ``GPC`` value.
+        Try to locate a product for the given GPC value.
+
+        :param session: Current session for Rattail DB.
+
+        :param gpc: :class:`~rattail.gpc.GPC` instance to match on.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
         """
+        from rattail.db.api.products import get_product_by_upc
+
         if not gpc:
             return
 
         # first we try the normal search
-        product = api.get_product_by_upc(session, gpc)
+        product = get_product_by_upc(session, gpc)
         if product:
             return product
 
         # maybe also try special search for "Type 2 UPC"
         if gpc.type2_upc and self.convert_type2_for_gpc_lookup():
-            cleaned = GPC('002{}00000'.format(gpc.data_str[1:6]),
-                          calc_check_digit='upc')
-            return api.get_product_by_upc(session, cleaned)
+            cleaned = self.make_gpc('002{}00000'.format(gpc.data_str[1:6]),
+                                    calc_check_digit='upc')
+            return get_product_by_upc(session, cleaned)
 
     def convert_type2_for_gpc_lookup(self):
-        return self.config.getbool('rattail', 'products.convert_type2_for_gpc_lookup',
-                                   default=False)
+        return self.config.getbool(
+            'rattail', 'products.convert_type2_for_gpc_lookup',
+            default=False)
+
+    def locate_product_for_vendor_code(self, session, entry, vendor=None, **kwargs):
+        """
+        Locate the product which matches the given vendor code.
+
+        This will do a lookup on the
+        :attr:`rattail.db.model.products.ProductCost.code` field only.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: Vendor item code value as string.
+
+        :param vendor: :class:`~rattail.db.model.vendors.Vendor` to
+           which to restrict the search.  While technically optional,
+           in most cases you should specify the vendor when doing this
+           type of lookup.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
+        """
+        from rattail.db.api.products import get_product_by_vendor_code
+
+        return get_product_by_vendor_code(session, entry, vendor=vendor)
+
+    def locate_product_for_alt_code(self, session, entry, **kwargs):
+        """
+        Locate the product which matches the given alternate code.
+
+        This will do a lookup on the
+        :attr:`rattail.db.model.products.ProductCode.code` field only.
+
+        :param session: Current session for Rattail DB.
+
+        :param entry: Alternate code value as string.
+
+        :returns: First :class:`~rattail.db.model.products.Product`
+           instance found if there was a match; otherwise ``None``.
+        """
+        from rattail.db.api.products import get_product_by_code
+
+        return get_product_by_code(session, entry)
+
+    def get_case_size(self, product):
+        """
+        Return the effective case size for the given product.
+        """
+        if product.case_size:
+            return product.case_size
+
+        cost = product.cost
+        if cost:
+            return cost.case_size
 
     def get_url(self, product, **kwargs):
         """
